/*******************************************************************************
 * Copyright (c) 2013, Perforce Software
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package com.perforce.search.test;

import java.util.List;

import org.springframework.mock.web.MockHttpServletRequest;
import org.springframework.mock.web.MockHttpServletResponse;
import org.springframework.web.servlet.ModelAndView;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.perforce.p4java.extension.server.Server;
import com.perforce.search.model.impl.SearchModelImpl;
import com.perforce.search.test.util.ScanningIndexTestBase;

public class SearchControllerTest extends ScanningIndexTestBase {

    @BeforeClass
    public void setUp() throws Exception {
	super.setUp();
    }

    @Test(enabled = false)
    public void testAuthorizedSearch() throws Exception {
	Server server = null;
	try {
	    // TODO: provide authenticated session
	    server = pool.acquire();
	    MockHttpServletRequest req = new MockHttpServletRequest();
	    MockHttpServletResponse resp = new MockHttpServletResponse();
	    SearchModelImpl searchModel = new SearchModelImpl();
	    searchModel.setQuery("perforce");
	    searchModel.setUserId(server.getLoginName());
	    searchModel.setTicket(server.getAuthTicket());
	    ModelAndView mav = searchController.authorizedSearch(req, resp,
		    searchModel);
	    Assert.assertEquals(mav.toString(), "this is a failing test");
	} finally {
	    pool.release(server);
	}
    }

    @Test
    public void testGetSearchFields() {
	Server server = null;
	try {
	    server = pool.acquire();
	    MockHttpServletRequest req = new MockHttpServletRequest();
	    MockHttpServletResponse resp = new MockHttpServletResponse();
	    SearchModelImpl searchModel = new SearchModelImpl();
	    searchModel.setQuery("perforce");
	    searchModel.setUserId(server.getLoginName());
	    searchModel.setTicket(server.getAuthTicket());
	    ModelAndView mav = searchController.getSearchFields(req, resp);
	    Assert.assertEquals(RootControllerTest.getStatusCode(mav), "200");
	    List<Object> payload = RootControllerTest.getList(mav, "payload");
	    Assert.assertNotEquals(payload, null);
	    // this varies with solr version, additional fields etc.
	    Assert.assertEquals(payload.size(), 65);
	} finally {
	    pool.release(server);
	}
    }

    @Test
    public void testSearch() {
	Server server = null;
	try {
	    server = pool.acquire();
	    MockHttpServletRequest req = new MockHttpServletRequest();
	    MockHttpServletResponse resp = new MockHttpServletResponse();
	    SearchModelImpl searchModel = new SearchModelImpl();
	    searchModel.setQuery("perforce");
	    searchModel.setUserId(server.getLoginName());
	    searchModel.setTicket(server.getAuthTicket());
	    ModelAndView mav = searchController.search(req, resp, searchModel);
	    List<Object> payload = RootControllerTest.getList(mav, "payload");
	    Assert.assertNotEquals(payload, null);
	    Assert.assertEquals(payload.size(), 6);
	} finally {
	    pool.release(server);
	}
    }

    @Test
    public void testSearchHEAD() {
	Server server = null;
	try {
	    server = pool.acquire();
	    MockHttpServletRequest req = new MockHttpServletRequest();
	    MockHttpServletResponse resp = new MockHttpServletResponse();
	    SearchModelImpl searchModel = new SearchModelImpl();
	    searchModel.setQuery("perforce");
	    searchModel.setUserId(server.getLoginName());
	    searchModel.setTicket(server.getAuthTicket());
	    ModelAndView mav = searchController.searchCommonsHead(req, resp);
	    Assert.assertEquals(RootControllerTest.getStatusCode(mav), "200");
	} finally {
	    pool.release(server);
	}
    }
}
